.control-label.field-with-error {
  color: $state-danger-text;
}

.form-control.field-with-error {
  border-color: $state-danger-text;
  @include box-shadow(inset 0 1px 1px rgba(0,0,0,.075)); // Redeclare so transitions work
  &:focus {
    border-color: darken($state-danger-text, 10%);
    $shadow: inset 0 1px 1px rgba(0,0,0,.075), 0 0 6px lighten($state-danger-text, 20%);
    @include box-shadow($shadow);
  }
}
